# -*- encoding: utf-8 -*-

##############################################################################
#
# Copyright (c) 2010-2012 E.Samyn - esamyn@gmail.com
# Author: E.Samyn
# www.erp-236.com
#
# This file is part of the project_specs module
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; 
# If not, see <http://www.gnu.org/licenses/> or  write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from mx import DateTime
import time

from osv import fields, osv

class task(osv.osv):
    _name = "project.task"
    _description = "Task"
    _inherit = "project.task"
        
    def _hours_from_specs(self, cr, uid, ids, names, arg, context=None):
        res = {}
        invoiced_hours = 0
        current_task = self.browse(cr, uid, ids)[0]
        for spec in current_task.specs_ids:
            if spec.delivery_qty:
                invoiced_hours += spec.delivery_qty
        res[current_task.id] = invoiced_hours
        return res
        
    _columns = {'specs_ids' : fields.one2many('project.specs', 'task_id', 'Specs'),
                'todo' : fields.text('TODO'),
                'invoiced_hours' : fields.function(_hours_from_specs, string='Invoiced hours', help="Computed from specs delivered quantities"),
    }

    
task()

#----- Spec Class
class project_specs(osv.osv):
    _name = "project.specs"
    _description = "specifications_management"
    _columns = {
        'task_id': fields.many2one('project.task', 'Task', ondelete='cascade'),
        'project_id' : fields.related('task_id','project_id',type="many2one",relation="project.project",string="project",store=False),
    # Main        
        'name' : fields.char("Title", size=250, required=True),
        'state' : fields.selection([('draft','Draft'),
                                ('tobeprecised','To be precised'),
                                ('valid','Valid'),
                                ('model','In modelisation'),
                                ('develop','In development'),
                                ('test','In test'),
                                ('done','Done'),
                                ('not_specified','Not specified'),
                                ('nextV','Next version'),
                                ('outoffield','Out of field'),
                                ('double','Doublon'),
                        ('canceled','Canceled'),], 'State'),
        'state_last_change__date' : fields.date("State last change date"),
    # Details
        'spec_date' : fields.date("Creation date"),
        'notincontract' : fields.boolean('Not in contrat'),
        'sequence': fields.integer('Sequence'),
        'spec_description' : fields.text('Customer requirement'),
    # Modelisation
        'model' : fields.text('Modelisation',help="Parametrage / modélisation"),
    # Dev
        'dev_history' : fields.text('Dev history'),
    # Test        
        'test_case' : fields.text('Reference test case'),
        'tests_ids' : fields.one2many('project.specs.tests','tests_id','Test'),
    # Delivery        
        'spec_history' : fields.text('Spec history'),
        'delivery_date' : fields.date("Delivery date"),
        'delivery_phase': fields.many2one('project.phase', 'Phase'),
        'delivery_qty': fields.float('Invoiced Hours',help="Invoiced our for this specification. Can be different from the real time spent"),
    }
    _defaults={
          'state': lambda *a: 'draft',
          'spec_date': lambda *a: time.strftime('%Y-%m-%d'),
    }
    _order = 'sequence,name asc'

project_specs()

#----- Test Class
class project_specs_tests(osv.osv):
    _name = "project.specs.tests"
    _description = "specs tests and history"
    _columns = {
        'tests_id' : fields.many2one('project.specs', 'tests',required=True),
        'name' : fields.char("Label", size=64,required=True),
        'test_person' : fields.char("Test person", size=64),
        'test_case' : fields.text("Test case"),
        'test_date' : fields.date("Test date"),
        'test_expected_result' : fields.text("Expected results"),
        'test_result' : fields.text("Real results"),
        'test_comments' : fields.text('Test comments'),
        'test_validated' : fields.boolean('Test valid'),
    }
project_specs_tests()
